package dao

import (
	"fmt"
	dbModel "sdk_api/model/db"

	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

func (d *Dao) CreateUser(user *dbModel.User) (err error) {
	return d.db.Clauses(clause.OnConflict{
		Columns:   []clause.Column{{Name: "user_id"}, {Name: "chat_id"}},
		DoUpdates: clause.Assignments(map[string]interface{}{"left_at": gorm.Expr("NULL")}),
	}).Create(user).Error
}

func (d *Dao) IncrMessageCount(a *dbModel.Active) (err error) {
	a.MsgCount = 1
	return d.db.Clauses(clause.OnConflict{
		Columns:   []clause.Column{{Name: "user_id"}, {Name: "chat_id"}, {Name: "unix_day"}},
		DoUpdates: clause.Assignments(map[string]interface{}{"msg_count": gorm.Expr(fmt.Sprintf("%s.msg_count + ?", a.TableName()), 1)}),
	}).Create(a).Error
}

func (d *Dao) GetUserActiveMsgCount(userId, chatId, unixDay int) (count int, err error) {
	a := dbModel.Active{}
	err = d.db.Model(&dbModel.Active{}).
		Where("user_id = ? AND chat_id = ? AND unix_day = ?", userId, chatId, unixDay).
		First(&a).Error

	if err == gorm.ErrRecordNotFound {
		return 0, nil
	}

	return a.MsgCount, err
}

func (d *Dao) UserExist(userId, chatId int) (exist bool, err error) {
	a := dbModel.User{}
	err = d.db.Model(&dbModel.User{}).
		Where("user_id = ? AND chat_id = ? AND left_at IS NULL", userId, chatId).
		First(&a).Error
	if err == gorm.ErrRecordNotFound {
		return false, nil
	}
	return err == nil, err
}

func (d *Dao) UserLeft(userId, chatId int) (err error) {
	return d.db.Model(&dbModel.User{}).
		Where("user_id = ? AND chat_id = ? AND left_at IS NULL", userId, chatId).
		Update("left_at", gorm.Expr("NOW()")).Error
}

func (d *Dao) IsSupportedChat(chatId int) (exist bool, err error) {
	a := dbModel.ChatGroup{}
	err = d.db.Model(&dbModel.ChatGroup{}).
		Where("chat_id = ?", chatId).
		First(&a).Error
	if err == gorm.ErrRecordNotFound {
		return false, nil
	}
	return err == nil, err
}

func (d *Dao) addTestGroup() (err error) {
	return d.db.Clauses(clause.OnConflict{DoNothing: true}).Create([]dbModel.ChatGroup{
		{
			ChatId:    -1002174503961,
			ChatTitle: "mail_notice",
		},
	}).Error
}
